==> aTunes_1.9.0.tar.gz: aTunes/src/net/sourceforge/atunes/kernel/handlers/RadioHandler.java <==
/*
 * aTunes 1.9.0
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.handlers;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ExecutionException;

import javax.swing.SwingWorker;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.modules.radio.Radio;
import net.sourceforge.atunes.kernel.modules.state.beans.ProxyBean;
import net.sourceforge.atunes.kernel.utils.NetworkUtils;
import net.sourceforge.atunes.kernel.utils.XMLUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class RadioHandler.
 * 
 * @author sylvain
 */
public class RadioHandler {

	/** The instance. */
	private static RadioHandler instance = new RadioHandler();

	/** The logger. */
	Logger logger = new Logger();

	/** The radios. */
	private List<Radio> radios;

	/** The preset radios. */
	private List<Radio> presetRadios;

	/** The retrieved preset radios. */
	List<Radio> retrievedPresetRadios = new ArrayList<Radio>();

	/** The no new stations. */
	boolean noNewStations = true;

	/**
	 * Instantiates a new radio handler.
	 */
	private RadioHandler() {
		// nothing to do
	}

	/**
	 * Gets the single instance of RadioHandler.
	 * 
	 * @return single instance of RadioHandler
	 */
	public static RadioHandler getInstance() {
		return instance;
	}

	/**
	 * Add the radio station from the add radio dialog.
	 */
	public void addRadio() {
		Radio radio = VisualHandler.getInstance().showAddRadioDialog();
		if (radio != null) {
			addRadio(radio);
		}
	}

	/**
	 * Add a radio station to the list.
	 * 
	 * @param radio
	 *            Station
	 */
	public void addRadio(Radio radio) {
		logger.info(LogCategories.HANDLER, "Adding radio");
		if (radio != null && !radios.contains(radio))
			radios.add(radio);
		Collections.sort(radios, Radio.getComparator());
		refresh();
	}

	/**
	 * Write stations to xml files.
	 */
	public void finish() {
		ApplicationDataHandler.getInstance().persistRadioCache(radios);
		// Only write preset list if new stations were added
		if (!noNewStations)
			ApplicationDataHandler.getInstance().persistPresetRadioCache(presetRadios);
	}

	/**
	 * Gets the radios.
	 * 
	 * @return the radios
	 */
	public List<Radio> getRadios() {
		return new ArrayList<Radio>(radios);
	}

	/**
	 * Gets the radio presets.
	 * 
	 * @return the preset radios, minus user maintained radio stations.
	 */
	public List<Radio> getRadioPresets() {
		// Check if new stations were added and set false if yes
		if (noNewStations == true) {
			noNewStations = presetRadios.containsAll(retrievedPresetRadios);
			retrievedPresetRadios.removeAll(presetRadios);
		} else
			// New stations were already found, so leave noNewStations as false
			retrievedPresetRadios.removeAll(presetRadios);
		presetRadios.addAll(retrievedPresetRadios);
		presetRadios.removeAll(radios);
		return new ArrayList<Radio>(presetRadios);
	}

	/**
	 * Gets the radios.
	 * 
	 * @param label
	 *            the label
	 * 
	 * @return the radios
	 */
	public List<Radio> getRadios(String label) {
		List<Radio> result = new ArrayList<Radio>();
		for (Radio radio : radios) {
			if (label.equals(radio.getLabel())) {
				result.add(radio);
			}
		}
		for (Radio radio : presetRadios) {
			if (label.equals(radio.getLabel())) {
				result.add(radio);
			}
		}
		refresh();
		return result;
	}

	/**
	 * Read radio stations lists. We use different files, one for presets which
	 * is not modified by the user and a second one for all the user
	 * modifications.
	 */
	void readRadios() {
		radios = ApplicationDataHandler.getInstance().retrieveRadioCache();
		presetRadios = ApplicationDataHandler.getInstance().retrieveRadioPreset();
	}

	/**
	 * Runnable process to read radio cache.
	 * 
	 * @return the read radios runnable
	 */
	public Runnable getReadRadiosRunnable() {
		return new Runnable() {
			@Override
			public void run() {
				readRadios();
			}
		};
	}

	/**
	 * Refreshes radio view.
	 */
	public void refresh() {
		ControllerProxy.getInstance().getNavigationController().refreshRadioTreeContent();
	}

	/**
	 * Remove a station from the list. Preset stations are not really removed
	 * but are marked so they not show up in the navigator
	 * 
	 * @param radio
	 *            Radio to be removed
	 */
	public void removeRadio(Radio radio) {
		logger.info(LogCategories.HANDLER, "Removing radio");
		if (!presetRadios.contains(radio)) {
			radios.remove(radio);
			refresh();
		}
		// Preset radio station, we can not delete from preset file directly but must mark it as removed.
		else {
			presetRadios.remove(radio);
			final Radio newRadio = new Radio(radio.getName(), radio.getUrl(), radio.getLabel());
			newRadio.setRemoved(true);
			radios.add(newRadio);
			refresh();
		}

	}

	/**
	 * Retrieve radios for browser.
	 * 
	 * @return the list< radio>
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@SuppressWarnings("unchecked")
	public List<Radio> retrieveRadiosForBrowser() throws IOException {
		try {
			String xml = NetworkUtils.readURL(NetworkUtils
					.getConnection(Constants.RADIO_LIST_DOWNLOAD_COMMON_JUKEBOX, NetworkUtils.getProxy(Kernel.getInstance().state.getProxy())));
			return (List<Radio>) XMLUtils.readObjectFromString(xml);
		} catch (Exception e) {
			String xml = NetworkUtils.readURL(NetworkUtils.getConnection(Constants.RADIO_LIST_DOWNLOAD, NetworkUtils.getProxy(Kernel.getInstance().state.getProxy())));
			return (List<Radio>) XMLUtils.readObjectFromString(xml);
		}
	}

	/**
	 * Retrieve radios.
	 */
	@SuppressWarnings("unchecked")
	/**
	 * Get radios from the internet (update preset list)
	 */
	public void retrieveRadios() {
		new SwingWorker<List<Radio>, Void>() {

			final ProxyBean proxy = Kernel.getInstance().state.getProxy();

			@Override
			protected List<Radio> doInBackground() throws Exception {
				String xml = NetworkUtils.readURL(NetworkUtils.getConnection(Constants.RADIO_LIST_DOWNLOAD, NetworkUtils.getProxy(proxy)));
				List<Radio> list = (List<Radio>) XMLUtils.readObjectFromString(xml);
				return list;
			}

			@Override
			protected void done() {
				try {
					retrievedPresetRadios.clear();
					retrievedPresetRadios.addAll(get());
					getRadioPresets();
					RadioHandler.getInstance().refresh();
				} catch (InterruptedException e) {
					logger.error(LogCategories.HANDLER, e);
				} catch (ExecutionException e) {
					logger.error(LogCategories.HANDLER, e);
				}

			}

		}.execute();
	}

	/**
	 * Change label of radio.
	 * 
	 * @param radioList
	 *            List of radios for which the label should be changed
	 * @param label
	 *            New label
	 */
	public void setLabel(List<Radio> radioList, String label) {
		for (Radio r : radioList) {
			// Write preset stations to user list in order to modify label
			if (presetRadios.contains(r)) {
				addRadio(r);
			}
			r.setLabel(label);
		}
	}

	/**
	 * Change radio name. Also permits changing name of preset stations.
	 * 
	 * @param radio
	 *            Radio station
	 * @param name
	 *            New name
	 */
	public void setName(Radio radio, String name) {
		if (presetRadios.contains(radio)) {
			addRadio(radio);
		}
		radio.setName(name);
	}
}

==> aTunes_1.10.1.tar.gz: aTunes/src/net/sourceforge/atunes/kernel/handlers/RadioHandler.java <==
/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.handlers;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ExecutionException;

import javax.swing.SwingWorker;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.kernel.ApplicationFinishListener;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.modules.radio.Radio;
import net.sourceforge.atunes.kernel.modules.state.beans.ProxyBean;
import net.sourceforge.atunes.kernel.utils.NetworkUtils;
import net.sourceforge.atunes.kernel.utils.XMLUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class RadioHandler.
 * 
 * @author sylvain
 */
public class RadioHandler implements ApplicationFinishListener {

	/** The instance. */
	private static RadioHandler instance = new RadioHandler();

	/** The logger. */
	Logger logger = new Logger();

	/** The radios. */
	private List<Radio> radios;

	/** The preset radios. */
	private List<Radio> presetRadios;

	/** The retrieved preset radios. */
	List<Radio> retrievedPresetRadios = new ArrayList<Radio>();

	/** The no new stations. */
	boolean noNewStations = true;

	/**
	 * Instantiates a new radio handler.
	 */
	private RadioHandler() {
		Kernel.getInstance().addFinishListener(this);
	}

	/**
	 * Gets the single instance of RadioHandler.
	 * 
	 * @return single instance of RadioHandler
	 */
	public static RadioHandler getInstance() {
		return instance;
	}

	/**
	 * Add the radio station from the add radio dialog.
	 */
	public void addRadio() {
		Radio radio = VisualHandler.getInstance().showAddRadioDialog();
		if (radio != null) {
			addRadio(radio);
		}
	}

	/**
	 * Add a radio station to the list.
	 * 
	 * @param radio
	 *            Station
	 */
	public void addRadio(Radio radio) {
		logger.info(LogCategories.HANDLER, "Adding radio");
		if (radio != null && !radios.contains(radio)) {
			radios.add(radio);
		}
		Collections.sort(radios, Radio.getComparator());
		refresh();
	}

	/**
	 * Write stations to xml files.
	 */
	public void applicationFinish() {
		ApplicationDataHandler.getInstance().persistRadioCache(radios);
		// Only write preset list if new stations were added
		if (!noNewStations) {
			ApplicationDataHandler.getInstance().persistPresetRadioCache(presetRadios);
		}
	}

	/**
	 * Gets the radios.
	 * 
	 * @return the radios
	 */
	public List<Radio> getRadios() {
		return new ArrayList<Radio>(radios);
	}

	/**
	 * Gets the radio presets.
	 * 
	 * @return the preset radios, minus user maintained radio stations.
	 */
	public List<Radio> getRadioPresets() {
		// Check if new stations were added and set false if yes
		if (noNewStations == true) {
			noNewStations = presetRadios.containsAll(retrievedPresetRadios);
			retrievedPresetRadios.removeAll(presetRadios);
		} else {
			// New stations were already found, so leave noNewStations as false
			retrievedPresetRadios.removeAll(presetRadios);
		}
		presetRadios.addAll(retrievedPresetRadios);
		presetRadios.removeAll(radios);
		return new ArrayList<Radio>(presetRadios);
	}

	/**
	 * Gets the radios.
	 * 
	 * @param label
	 *            the label
	 * 
	 * @return the radios
	 */
	public List<Radio> getRadios(String label) {
		List<Radio> result = new ArrayList<Radio>();
		for (Radio radio : radios) {
			if (label.equals(radio.getLabel())) {
				result.add(radio);
			}
		}
		for (Radio radio : presetRadios) {
			if (label.equals(radio.getLabel())) {
				result.add(radio);
			}
		}
		refresh();
		return result;
	}

	/**
	 * Read radio stations lists. We use different files, one for presets which
	 * is not modified by the user and a second one for all the user
	 * modifications.
	 */
	void readRadios() {
		radios = ApplicationDataHandler.getInstance().retrieveRadioCache();
		presetRadios = ApplicationDataHandler.getInstance().retrieveRadioPreset();
	}

	/**
	 * Runnable process to read radio cache.
	 * 
	 * @return the read radios runnable
	 */
	public Runnable getReadRadiosRunnable() {
		return new Runnable() {
			@Override
			public void run() {
				readRadios();
			}
		};
	}

	/**
	 * Refreshes radio view.
	 */
	public void refresh() {
		ControllerProxy.getInstance().getNavigationController().refreshRadioTreeContent();
	}

	/**
	 * Remove a station from the list. Preset stations are not really removed
	 * but are marked so they not show up in the navigator
	 * 
	 * @param radio
	 *            Radio to be removed
	 */
	public void removeRadio(Radio radio) {
		logger.info(LogCategories.HANDLER, "Removing radio");
		if (!presetRadios.contains(radio)) {
			radios.remove(radio);
			refresh();
		}
		// Preset radio station, we can not delete from preset file directly but must mark it as removed.
		else {
			presetRadios.remove(radio);
			final Radio newRadio = new Radio(radio.getName(), radio.getUrl(), radio.getLabel());
			newRadio.setRemoved(true);
			radios.add(newRadio);
			refresh();
		}

	}

	/**
	 * Retrieve radios for browser.
	 * 
	 * @return the list< radio>
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	@SuppressWarnings("unchecked")
	public List<Radio> retrieveRadiosForBrowser() throws IOException {
		try {
			String xml = NetworkUtils.readURL(NetworkUtils
					.getConnection(Constants.RADIO_LIST_DOWNLOAD_COMMON_JUKEBOX, NetworkUtils.getProxy(Kernel.getInstance().state.getProxy())));
			return (List<Radio>) XMLUtils.readObjectFromString(xml);
		} catch (Exception e) {
			String xml = NetworkUtils.readURL(NetworkUtils.getConnection(Constants.RADIO_LIST_DOWNLOAD, NetworkUtils.getProxy(Kernel.getInstance().state.getProxy())));
			return (List<Radio>) XMLUtils.readObjectFromString(xml);
		}
	}

	/**
	 * Retrieve radios.
	 */
	@SuppressWarnings("unchecked")
	/*
	 * Get radios from the internet (update preset list)
	 */
	public void retrieveRadios() {
		new SwingWorker<List<Radio>, Void>() {

			final ProxyBean proxy = Kernel.getInstance().state.getProxy();

			@Override
			protected List<Radio> doInBackground() throws Exception {
				String xml = NetworkUtils.readURL(NetworkUtils.getConnection(Constants.RADIO_LIST_DOWNLOAD, NetworkUtils.getProxy(proxy)));
				List<Radio> list = (List<Radio>) XMLUtils.readObjectFromString(xml);
				return list;
			}

			@Override
			protected void done() {
				try {
					retrievedPresetRadios.clear();
					retrievedPresetRadios.addAll(get());
					getRadioPresets();
					RadioHandler.getInstance().refresh();
				} catch (InterruptedException e) {
					logger.error(LogCategories.HANDLER, e);
				} catch (ExecutionException e) {
					logger.error(LogCategories.HANDLER, e);
				}

			}

		}.execute();
	}

	/**
	 * Change label of radio.
	 * 
	 * @param radioList
	 *            List of radios for which the label should be changed
	 * @param label
	 *            New label
	 */
	public void setLabel(List<Radio> radioList, String label) {
		for (Radio r : radioList) {
			// Write preset stations to user list in order to modify label
			if (presetRadios.contains(r)) {
				addRadio(r);
			}
			r.setLabel(label);
		}
	}

	/**
	 * Change radio name. Also permits changing name of preset stations.
	 * 
	 * @param radio
	 *            Radio station
	 * @param name
	 *            New name
	 */
	public void setName(Radio radio, String name) {
		if (presetRadios.contains(radio)) {
			addRadio(radio);
		}
		radio.setName(name);
	}
}
